#!/bin/bash
#
# Copyright (C) 2004-2013 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of le-save-config
#
# le-save-config is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

#ASK FOR ROOT
if [ $(id -u) -ne 0 ]; then
    echo "This script must be run as root!"
    gksu -S -l -m "É necessário ter permissão de superusuário. Digite a senha:" "sh /usr/sbin/le-italc-config.sh"
    exit $?
fi

#GLOBAL VARIABLES
#Script path for monoterminal:
SCRIPT_MONO_PATH_SRC="/etc/le-italc-config/ica-start-monoterminal.sh"
SCRIPT_MONO_PATH_DST="/etc/lightdm/greeter-exec-scripts/ica-start-monoterminal.sh"

#Script path for multiterminal:
SCRIPT_MULT_PATH_SRC="/etc/le-italc-config/ica-start-multiterminal.sh"
SCRIPT_MULT_PATH_DST="/etc/profile.d/ica-start-multiterminal.sh"


#=================================UNCONFIG-FUNCTIONS===============================

unconfig_monoterminal () {
    if [ ! -f "$SCRIPT_MONO_PATH_DST" ]; then
        zenity --warning --title="Aviso" --text="Não foi identificada nenhuma configuração monoterminal"
        return 1
    else
        #Remove key and script
        rm -f /etc/italc/public/teacher/key
        rm -f "$SCRIPT_MONO_PATH_DST"
        return 0
    fi
}

unconfig_multiterminal () {
    if [ ! -f "$SCRIPT_MULT_PATH_DST" ]; then
        zenity --warning --title="Aviso" --text="Não foi identificada nenhuma configuração multiterminal"
        return 1
    else
        #Remove key and script
        rm -f "/etc/italc/public/teacher/key"
        rm -f "$SCRIPT_MULT_PATH_DST"
        return 0
    fi
    #This function does not remove computers entries from italc.
}
#====================================================================================

#==WELCOME===========================================================
zenity --question --title="Configurador do iTalc" \
       --height=100 --width=600 \
       --text="Bem vindo ao auto configurador do iTalc\! \
          \n\nExecute esse configurador primeiramente na máquina designada para ser o servidor do iTalc e em seguida nos terminais clientes. No caso de estar em um multiterminal apenas uma execução desse instalador é necessária para configuração da estação cliente e servidor. \
          \n\nClique em Continuar para prosseguir com a configuração" \
          --ok-label="Continuar" --cancel-label="Sair"
[ $? -eq 1 ] && exit 0 #Cancel Pressed
#====================================================================

#Configuration Variables Names:
# (Values: 1-True 0-False)
#
#  isMult = multiterminal or 'monoterminal'?
#  isClient = Client or Server? 

#QUESTIONS

#If already exist a configuration
if [ -f "$SCRIPT_MONO_PATH_DST" -o -f "$SCRIPT_MULT_PATH_DST" ]; then
    #ASK: Is it configuration?
    zenity --question --title="Configurador do iTalc" \
           --text="Foi detectada uma configuração prévia do iTalc nessa máquina.
                   \nDeseja continuar? \
                   \nTodas as configurações anteriores serão desfeitas" \
           --ok-label="Sim" --cancel-label="Não"
    [ $? -eq 1 ] && exit 0
fi

#ASK: Is it multiterminal?
zenity --question --title="Configurador do iTalc" \
       --height=100 --width=450 \
       --text="Qual é o tipo do terminal?" \
       --ok-label="Multiterminal" --cancel-label="Monoterminal"
if [ $? -eq 0 ]; then #MULTITERMINAL
    isMult=1
    #ASK: How much terminals there is(for iTalc auto-adder)
    numTerm=$(
        zenity --title="Número de terminais" --height=330 \
            --text="Selecione o número de terminais do seu multiterminal:" \
            --list --radiolist \
            --column="Selecionado" \
            --column="Número de terminais" \
           "TRUE" "1" "" "2" "" "3" "" "4" "" "5" "" "6" "" "7" "" "8" "" "9"
    )
    if [ $? -eq 1 ]; then
        echo "Operação Cancelada"
        exit 4
    fi
else #MONOTERMINAL
    isMult=0
    #If it is MONOterminal it can be Client or Master configuration
    #ASK: Master or Client machine/terminal?
    zenity --question --title="Configurador do iTalc" \
           --height=100 --width=450 \
           --text="Esta estação será usada como cliente ou servidor do iTalc?" \
           --ok-label="Cliente" --cancel-label="Servidor"
    if [ $? -eq 0 ]; then #Sim
        isClient=1
    else
        isClient=0
    fi
fi

#ASK: Confirm
[ $isMult   -eq 1 ] && s1="Multiterminal" || s1="Monoterminal"
[ $isClient -eq 1 ] && s2="Cliente" ||  s2="Servidor"
[ $isMult   -eq 1 ] && s2="Todas"

zenity --question --title="Configurador do iTalc" \
       --text="Verifique as informações: \
              \nTipo: $s1 \
              \nFunção: $s2" \
       --ok-label="Confirmar" --cancel-label="Cancelar"
[ $? -eq 1 ] && exit 0

#If confirmed, remove old configurations if exists
if [ -f "$SCRIPT_MONO_PATH_DST" ]; then
    unconfig_monoterminal
elif [ -f "$SCRIPT_MULT_PATH_DST" ]; then #Multiteminal
    unconfig_multiterminal
fi

##===================================================================================================================##
##---START_FUNCTIONS-------------------------------------------------------------------------------------------------##
##===================================================================================================================##

copy_script_monoterminal () {
    cp "$SCRIPT_MONO_PATH_SRC" "$SCRIPT_MONO_PATH_DST"
    chmod +x "$SCRIPT_MONO_PATH_DST"
}

copy_script_multiterminal () {
    cp "$SCRIPT_MULT_PATH_SRC" "$SCRIPT_MULT_PATH_DST"
    chmod +x "$SCRIPT_MULT_PATH_DST"
}

config_client_monoterminal () {
    #Steps:
    ## 1. Ask for keys
    ## 2. Import keys
    zenity --question --title="Configurador do iTalc" \
        --text="Para configurar o cliente é necessário ter sido configurada a estação servidora e adquirido o arquivo de chave do italc.\n\nClique em Selecionar arquivo e selecione o arquivo de chave." \
        --ok-label="Selecionar Arquivo" --cancel-label="Sair"
    [ $? -eq 1 ] && exit 0

    while [ 1 ]; do
        local key_file="$(zenity --file-selection)"
        local key_filename="$( basename "$key_file")"
        imc -importpublickey "$key_file"
        if [ $? -eq 255 ]; then # Imc invalid keyfile error code
            zenity --question --title="Configurador do iTalc" --text="Arquivo inválido." \
                   --height=150 --width=250 \
                   --ok-label="Selecionar outro arquivo" --cancel-label="Sair"
            [ $? -eq 1 ] && exit 0
            continue
        fi
        break
    done

    chmod 755 /etc/italc/public -R

    copy_script_monoterminal #copy ica autorun script
}

config_master_monoterminal () {
    local tmp_dir_chave="$(mktemp -d)"
    imc -role teacher -createkeypair 1>/dev/null

    #Protect private keys
    chown $SUDO_USER /etc/italc/keys/ -R
    sudo -u $SUDO_USER chmod 700 /etc/italc/keys/private -R
    #Release public key
    sudo -u $SUDO_USER chmod 755 /etc/italc/keys/public -R    

    #Show folder with the key to copy
    cp /etc/italc/keys/public/teacher/key $tmp_dir_chave/Chave-Italc

    #Give key file to user
    chown $SUDO_USER $tmp_dir_chave -R

    sudo -u $SUDO_USER xdg-open $tmp_dir_chave & # I chose to use xdg-open for a broader compatibility
    sleep 2 #time to open file browser
    zenity --info --title="Configurador do iTalc" --text="Copie o arquivo Chave-Italc para as estações clientes, ele será solicitado pelo configurador."

    copy_script_monoterminal #copy ica autorun script
}
config_multiterminal () {

    #Generate public and private keys
    imc -role teacher -createkeypair

    #Protect private keys
    chown professor /etc/italc/keys/ -R
    sudo -u professor chmod 700 /etc/italc/keys/private -R
    #Release public key
    sudo -u professor chmod 755 /etc/italc/keys/public -R
   
    #Set config and create autorun script
    imc -setconfigvalue Authentication/LogonAuthenticationEnabled=0 #Don't ask for login to enter italc. Only teacher can read private key.
    copy_script_multiterminal
    #Give professor right to write to configuration file.
    chown professor /etc/xdg/iTALC\ Solutions/iTALC.conf 
    chmod 0600 /etc/xdg/iTALC\ Solutions/iTALC.conf

    if (who --users | awk -F " " '{print $1}' | grep -q "^professor$"); then #If teacher is logged in execute autorun script 
        su professor -c ".${SCRIPT_MULT_PATH_DST}" #Run script, so teacher doesn't need to relogin
    fi

}
add_computers_multiterminal () {
    local num_of_computers="$1"
    local configFile="/home/professor/.italc/GlobalConfig.xml"
    local configFileDir="/home/professor/.italc/"

    #Configuration File exists?
    if [ ! -f "$configFile" ]; then
        mkdir -p $configFileDir
        touch $configFile
    fi
    bool_xml="0" # Is xml header missing?
    grep -q "<?xml version=" $configFile;
    if [ $? -eq 1 ]; then
        echo "<?xml version=\"1.0\"?>" >> $configFile
        echo "<!DOCTYPE italc-config-file>"  >> $configFile
        echo "<globalclientconfig version=\"2.0.1\">" >> $configFile
        echo "  <body>" >> $configFile
        bool_xml=1
        CLASSR_NUM=1
    else
        # If xml exist resolve a name for classroom that does not match any already there
        for CLASSR_NUM in 1 2 3 4 5 6 7 8 9; do
            grep -q "Sala De Aula $CLASSR_NUM" $configFile
            [ $? -ne 0 ] && break
        done
        ##Remove ending
        sed -i '/<\/body>/d' $configFile
        sed -i '/<\/globalclientconfig>/d' $configFile
    fi

    ##Add Class and Hostnames
    echo "    <classroom name=\"Sala De Aula $CLASSR_NUM\">" >> $configFile
    for i in $(seq 1 $num_of_computers); do #add even the professor(since we don't know which display he will be at)
        rand_num="$(tr -cd 0-9 </dev/urandom | head -c 6)" #Get random number of 6 digit
        echo "      <client hostname=\"localhost:1100$i\" mac=\"\" type=\"0\" id=\"$rand_num\" name=\"Estação $i\"/>" >> $configFile
    done
    echo "    </classroom>" >> $configFile

    ##Refinish the file
    echo "  </body>" >> $configFile
    echo "</globalclientconfig>" >> $configFile

}
showHelp () {
    #Show help page
    su $SUDO_USER -c "xdg-open /usr/local/share/doc/le-save-config/help.html"
    return 0
}

##===================================================================================================================##
##---END_FUNCTIONS---------------------------------------------------------------------------------------------------##
##===================================================================================================================##

#Call right configuration function
if [ $isMult -eq 0 ]; then
    if [ $isClient -eq 1 ]; then
        config_client_monoterminal
    else
        config_master_monoterminal
    fi
elif [ $isMult -eq 1 ]; then
    config_multiterminal
    add_computers_multiterminal "$numTerm"

    zenity --question --height=150 --width=450 \
    --title="Multiterminal configurado!" \
    --text="AVISO: O usuário professor deve usar a estação conectada na placa onboard. \
    \n\nPara que todas as configurações tomem efeito é necessário relogar todos os usuários alunos. \
    \n\nPara abrir o italc, execute o comando 'italc' no terminal ou abra o iTalc pelo menu de aplicativos." \
    --ok-label="Concluir" --cancel-label="Ajuda"
    [ $? -eq 1 ] && showHelp || exit 0
fi
